package main

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"net/url"
	"regexp"
	"strings"
	"time"
)

// Config holds configuration from config.json
type Config struct {
	PushAPI        string            `json:"push_api"`
	PushKey        string            `json:"push_key"`
	CourseStart    string            `json:"course_start"`    // Format: YYYY-MM-DD
	ReloadInterval float64           `json:"reload_interval"` // In hours
	ClassTimes     map[string]string `json:"class_times"`     // Period to start time (e.g., "1": "08:00")
}

// Course represents a course entry from courses.json
type Course struct {
	Kcmc  string `json:"kcmc"`  // Course name
	Sksj  string `json:"sksj"`  // Schedule (e.g., "星期五第1-2节{1-19周(单)}")
	Jxdd  string `json:"jxdd"`  // Location
	Skfs  string `json:"skfs"`  // Teaching method
	Rwzxs string `json:"rwzxs"` // Duration (optional)
}

// CoursesJSON represents the structure of courses.json
type CoursesJSON struct {
	Items []Course `json:"items"`
}

// CourseSchedule represents a parsed course schedule
type CourseSchedule struct {
	DayOfWeek   int    // 1=Monday, 7=Sunday
	Periods     []int  // e.g., [1, 2] for periods 1-2
	Weeks       []int  // e.g., [1, 3, 5] for odd weeks
	CourseName  string // Course name
	Location    string // Location
	DurationMin int    // Duration in minutes (optional)
}

func main() {
	// 设置时区为东八区
	loc, err := time.LoadLocation("Asia/Shanghai")
	if err != nil {
		log.Fatalf("加载时区失败: %v", err)
	}
	time.Local = loc

	// Load configuration
	config, err := loadConfig()
	if err != nil {
		log.Fatalf("加载配置失败: %v", err)
	}
	startDate, err := time.Parse("2006-01-02", config.CourseStart)
	if err != nil {
		log.Fatalf("解析课程开始日期失败: %v", err)
	}
	startDate = startDate.In(loc)

	// Load initial courses
	courses, err := loadCourses()
	if err != nil {
		log.Fatalf("加载课程失败: %v", err)
	}
	fmt.Printf("课程加载成功，共有 %d 门课程\n", len(courses.Items))

	// Parse schedules
	var schedules []CourseSchedule
	for _, course := range courses.Items {
		if course.Skfs == "网络教学" {
			continue
		}
		scheds, err := parseSchedule(course.Sksj, course.Kcmc, course.Jxdd, course.Rwzxs)
		if err != nil {
			log.Printf("解析课程 %s 失败: %v", course.Kcmc, err)
			continue
		}
		schedules = append(schedules, scheds...)
	}

	// Print initial schedule
	currentWeek := getWeekNumber(startDate, time.Now())
	printSchedule(schedules, currentWeek, time.Now(), config.ClassTimes)

	// Track last printed schedule to avoid duplicates
	lastPrinted := make(map[string]bool)
	for _, s := range schedules {
		key := fmt.Sprintf("%s-%d-%v", s.CourseName, s.DayOfWeek, s.Periods)
		lastPrinted[key] = true
	}

	// Timers
	reloadTicker := time.NewTicker(time.Duration(config.ReloadInterval * float64(time.Hour)))
	defer reloadTicker.Stop()
	checkTicker := time.NewTicker(time.Minute)
	defer checkTicker.Stop()

	for {
		select {
		case t := <-checkTicker.C:
			currentWeek = getWeekNumber(startDate, t)
			// Check for forced printing at 7:30 AM and 12:30 PM
			if (t.Hour() == 7 && t.Minute() == 30) || (t.Hour() == 12 && t.Minute() == 30) {
				fmt.Printf("\n[%s] 强制打印课程安排:\n", t.Format("2006-01-02 15:04:05"))
				printSchedule(schedules, currentWeek, t, config.ClassTimes)
				for k := range lastPrinted {
					lastPrinted[k] = true
				}
			}

			// Check for notifications
			for _, s := range schedules {
				if !contains(s.Weeks, currentWeek) || s.DayOfWeek != int(t.Weekday()) {
					continue
				}
				startTime := getCourseStartTime(s.Periods[0], t, config.ClassTimes)
				notifyTime := startTime.Add(-20 * time.Minute)

				// 调试日志
				if t.Minute()%10 == 0 { // 每10分钟打印一次调试信息
					log.Printf("检查课程: %s, 当前时间: %v, 推送时间: %v, 匹配: %v",
						s.CourseName, t.Format("15:04:05"), notifyTime.Format("15:04:05"),
						t.Hour() == notifyTime.Hour() && t.Minute() == notifyTime.Minute())
				}

				if t.Hour() == notifyTime.Hour() && t.Minute() == notifyTime.Minute() {
					title := fmt.Sprintf("课程提醒: %s", s.CourseName)
					message := fmt.Sprintf("**课程**: %s\n**时间**: %s %s\n**地点**: %s",
						s.CourseName, t.Format("2006-01-02"), startTime.Format("15:04"), s.Location)
					log.Printf("尝试发送通知: %s", title)
					if err := sendPushNotification(config.PushAPI, config.PushKey, title, message); err != nil {
						log.Printf("发送通知失败: %v", err)
					}
				}
			}

		case t := <-reloadTicker.C:
			// Reload courses
			newCourses, err := loadCourses()
			if err != nil {
				log.Printf("重载课程失败: %v", err)
				continue
			}
			newSchedules := []CourseSchedule{}
			for _, course := range newCourses.Items {
				if course.Skfs == "网络教学" {
					continue
				}
				scheds, err := parseSchedule(course.Sksj, course.Kcmc, course.Jxdd, course.Rwzxs)
				if err != nil {
					log.Printf("解析课程 %s 失败: %v", course.Kcmc, err)
					continue
				}
				newSchedules = append(newSchedules, scheds...)
			}
			// Check if schedule changed
			changed := false
			newPrinted := make(map[string]bool)
			for _, s := range newSchedules {
				key := fmt.Sprintf("%s-%d-%v", s.CourseName, s.DayOfWeek, s.Periods)
				newPrinted[key] = true
				if !lastPrinted[key] {
					changed = true
				}
			}
			for k := range lastPrinted {
				if !newPrinted[k] {
					changed = true
				}
			}
			if changed || len(newSchedules) != len(schedules) {
				schedules = newSchedules
				fmt.Printf("\n[%s] 课程重载成功，共有 %d 门课程\n", t.Format("2006-01-02 15:04:05"), len(schedules))
				currentWeek = getWeekNumber(startDate, t)
				printSchedule(schedules, currentWeek, t, config.ClassTimes)
				lastPrinted = newPrinted
			} else {
				log.Println("课程数据未变更，跳过打印")
			}
		}
	}
}

// loadConfig loads configuration from config.json
func loadConfig() (Config, error) {
	var config Config
	data, err := ioutil.ReadFile("config.json")
	if err != nil {
		return config, fmt.Errorf("读取 config.json 失败: %v", err)
	}
	if err := json.Unmarshal(data, &config); err != nil {
		return config, fmt.Errorf("解析 config.json 失败: %v", err)
	}
	if config.ReloadInterval < 0.1 || config.ReloadInterval > 48 {
		return config, fmt.Errorf("重载间隔必须在 0.1 到 48 小时之间")
	}
	if config.ClassTimes == nil {
		return config, fmt.Errorf("config.json 中缺少 class_times 配置")
	}
	for period, timeStr := range config.ClassTimes {
		if _, err := time.Parse("15:04", timeStr); err != nil {
			return config, fmt.Errorf("无效的 class_times 格式，期间 %s: %s", period, timeStr)
		}
	}
	return config, nil
}

// loadCourses loads courses from courses.json
func loadCourses() (CoursesJSON, error) {
	var courses CoursesJSON
	data, err := ioutil.ReadFile("courses.json")
	if err != nil {
		return courses, fmt.Errorf("读取 courses.json 失败: %v", err)
	}
	if err := json.Unmarshal(data, &courses); err != nil {
		return courses, fmt.Errorf("解析 courses.json 失败: %v", err)
	}
	return courses, nil
}

// parseSchedule parses the sksj field into CourseSchedule
func parseSchedule(sksj, courseName, location, rwzxs string) ([]CourseSchedule, error) {
	var schedules []CourseSchedule
	// Split multiple schedules (e.g., "星期二第1-2节{1-14周,16-20周};...")
	parts := strings.Split(sksj, ";")
	dayMap := map[string]int{
		"星期一": 1, "星期二": 2, "星期三": 3, "星期四": 4,
		"星期五": 5, "星期六": 6, "星期日": 7,
	}
	periodRegex := regexp.MustCompile(`第(\d+)(?:-(\d+))?节`)
	weekRegex := regexp.MustCompile(`\{([^}]+)\}`)
	for _, part := range parts {
		part = strings.TrimSpace(part)
		if part == "" {
			continue
		}
		// Extract day
		var day int
		for dayName, dayNum := range dayMap {
			if strings.HasPrefix(part, dayName) {
				day = dayNum
				part = strings.TrimPrefix(part, dayName)
				break
			}
		}
		if day == 0 {
			log.Printf("无法解析星期: %s", part)
			continue
		}
		// Extract periods
		periodMatch := periodRegex.FindStringSubmatch(part)
		if len(periodMatch) < 2 {
			log.Printf("无法解析节次: %s", part)
			continue
		}
		startPeriod, err := parseInt(periodMatch[1])
		if err != nil {
			log.Printf("节次解析错误: %v", err)
			continue
		}
		endPeriod := startPeriod
		if len(periodMatch) > 2 && periodMatch[2] != "" {
			endPeriod, err = parseInt(periodMatch[2])
			if err != nil {
				log.Printf("节次解析错误: %v", err)
				continue
			}
		}
		var periods []int
		for p := startPeriod; p <= endPeriod; p++ {
			periods = append(periods, p)
		}
		// Extract weeks
		weekMatch := weekRegex.FindStringSubmatch(part)
		if len(weekMatch) < 2 {
			log.Printf("无法解析周次: %s", part)
			continue
		}
		weeks, singleDouble, err := parseWeeks(weekMatch[1])
		if err != nil {
			log.Printf("周次解析错误: %v", err)
			continue
		}
		// Handle duration
		duration := 0
		if rwzxs != "" {
			duration, err = parseInt(rwzxs)
			if err != nil {
				log.Printf("课时解析错误: %v", err)
			}
		}
		schedule := CourseSchedule{
			DayOfWeek:   day,
			Periods:     periods,
			Weeks:       weeks,
			CourseName:  courseName,
			Location:    location,
			DurationMin: duration,
		}
		// Filter by single/double weeks
		if singleDouble == "单" {
			var oddWeeks []int
			for _, w := range weeks {
				if w%2 == 1 {
					oddWeeks = append(oddWeeks, w)
				}
			}
			schedule.Weeks = oddWeeks
		} else if singleDouble == "双" {
			var evenWeeks []int
			for _, w := range weeks {
				if w%2 == 0 {
					evenWeeks = append(evenWeeks, w)
				}
			}
			schedule.Weeks = evenWeeks
		}
		schedules = append(schedules, schedule)
	}
	return schedules, nil
}

// parseWeeks parses week ranges (e.g., "1-14周,16-20周(单)")
func parseWeeks(weekStr string) ([]int, string, error) {
	var weeks []int
	var singleDouble string
	// Check for single/double week indicator
	if strings.Contains(weekStr, "(单)") {
		singleDouble = "单"
		weekStr = strings.Replace(weekStr, "(单)", "", 1)
	} else if strings.Contains(weekStr, "(双)") {
		singleDouble = "双"
		weekStr = strings.Replace(weekStr, "(双)", "", 1)
	}
	// Split ranges
	ranges := strings.Split(weekStr, ",")
	for _, r := range ranges {
		r = strings.TrimSuffix(r, "周")
		if strings.Contains(r, "-") {
			parts := strings.Split(r, "-")
			if len(parts) != 2 {
				return nil, "", fmt.Errorf("无效的周次范围: %s", r)
			}
			start, err := parseInt(parts[0])
			if err != nil {
				return nil, "", err
			}
			end, err := parseInt(parts[1])
			if err != nil {
				return nil, "", err
			}
			for i := start; i <= end; i++ {
				weeks = append(weeks, i)
			}
		} else {
			week, err := parseInt(r)
			if err != nil {
				return nil, "", err
			}
			weeks = append(weeks, week)
		}
	}
	return weeks, singleDouble, nil
}

// parseInt is a helper to parse string to int
func parseInt(s string) (int, error) {
	var n int
	_, err := fmt.Sscanf(s, "%d", &n)
	return n, err
}

// getWeekNumber calculates the current week number based on course start date
func getWeekNumber(startDate time.Time, now time.Time) int {
	days := int(now.Sub(startDate).Hours() / 24)
	week := days/7 + 1
	if week < 1 {
		return 1
	}
	return week
}

// getCourseStartTime returns the start time for a given period
func getCourseStartTime(period int, date time.Time, classTimes map[string]string) time.Time {
	// Look up the start time for the period
	periodStr := fmt.Sprintf("%d", period)
	timeStr, ok := classTimes[periodStr]
	if !ok {
		log.Printf("未找到期间 %d 的开始时间，使用默认时间 08:00", period)
		timeStr = "08:00"
	}
	// Parse the time
	t, err := time.Parse("15:04", timeStr)
	if err != nil {
		log.Printf("解析时间 %s 失败，使用默认时间 08:00: %v", timeStr, err)
		t, _ = time.Parse("15:04", "08:00")
	}
	return time.Date(date.Year(), date.Month(), date.Day(), t.Hour(), t.Minute(), 0, 0, date.Location())
}

// sendPushNotification sends a push notification via the API
func sendPushNotification(api, pushkey, title, message string) error {
	url := fmt.Sprintf("%s?pushkey=%s&text=%s&desp=%s&type=markdown", api, url.QueryEscape(pushkey), url.QueryEscape(title), url.QueryEscape(message))
	resp, err := http.Get(url)
	if err != nil {
		return fmt.Errorf("推送通知失败: %v", err)
	}
	defer resp.Body.Close()

	body, _ := ioutil.ReadAll(resp.Body)
	log.Printf("推送API响应: 状态码=%d, 内容=%s", resp.StatusCode, string(body))

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("推送通知返回非200状态码: %d", resp.StatusCode)
	}
	log.Printf("推送通知成功: %s", title)
	return nil
}

// printSchedule prints the course schedule in Chinese
func printSchedule(schedules []CourseSchedule, currentWeek int, now time.Time, classTimes map[string]string) {
	fmt.Printf("当前时间: %s\n", now.Format("2006年01月02日 15:04:05"))
	fmt.Printf("当前周次 | 第%d周\n", currentWeek)
	fmt.Println("本周课程安排:")
	for _, s := range schedules {
		if !contains(s.Weeks, currentWeek) {
			continue
		}
		startTime := getCourseStartTime(s.Periods[0], now, classTimes)
		periodStr := fmt.Sprintf("第%d-%d节", s.Periods[0], s.Periods[len(s.Periods)-1])
		weekStr := strings.Join(intSliceToString(s.Weeks), ",")
		fmt.Printf("课程: %s | 时间: 星期%d %s (%s) | 周次: %s | 地点: %s | 推送时间: %s\n",
			s.CourseName, s.DayOfWeek, periodStr, startTime.Format("15:04"), weekStr, s.Location,
			startTime.Add(-20*time.Minute).Format("15:04"))
	}
}

// contains checks if a slice contains a value
func contains(slice []int, val int) bool {
	for _, v := range slice {
		if v == val {
			return true
		}
	}
	return false
}

// intSliceToString converts int slice to string slice
func intSliceToString(slice []int) []string {
	var result []string
	for _, v := range slice {
		result = append(result, fmt.Sprintf("%d", v))
	}
	return result
}
